﻿--------------------------------------------------------------------------------------------------------
--                                             AceAddon init                                          --
--------------------------------------------------------------------------------------------------------
Broker_Wintergrasper = LibStub("AceAddon-3.0"):NewAddon("Broker_Wintergrasper", "AceEvent-3.0", "AceTimer-3.0")
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local L = LibStub("AceLocale-3.0"):GetLocale("Broker_Wintergrasper")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")

--------------------------------------------------------------------------------------------------------
--                           Broker_Wintergrasper variables and defaults                              --
--------------------------------------------------------------------------------------------------------
local status = nil
local faction = nil
local nextBattle = nil

local DATABASE_DEFAULTS = {
	profile = {
		showSeconds = false,
		tellSelf = true,
		tellGuild = true,
		tellParty = false,
		tellRaid = false,
		timeNow = false,
		timeFive = true,
		timeFifteen = true,
		timeThirty = true,
	},
}

--------------------------------------------------------------------------------------------------------
--                              Broker_Wintergrasper options pannel                                   --
--------------------------------------------------------------------------------------------------------
Broker_Wintergrasper.options = {
	type = "group",
	name = "Broker Wintergrasper",
	args = {
		general = {
			order = 1,
			type = "group",
			name = L["General Settings"],
			cmdInline = true,
			args = {
				seconds = {
					order = 1,
					type = "toggle",
					name = L["Show seconds"],
					desc = L["Toggle to show seconds."],
					get = function()
						return profileDB.showSeconds
					end,
					set = function(key, value)
						profileDB.showSeconds = value
					end,
				},
				announceTo = {
					order = 2,
					type = 'group',
					name = L["Announce to"],
					desc = L["Choose whom to announce when battle will start."],
					args = {
						self = {
							order = 1,
							type = 'toggle',
							name = L["Self"],
							desc = L["Warn self on upcoming battle."],
							get = function() return profileDB.tellSelf end,
							set = function(key, value) profileDB.tellSelf = value end,
						},
						guild = {
							order = 2,
							type = 'toggle',
							name = L["Guild"],
							desc = L["Warn guild on upcoming battle."],
							get = function() return profileDB.tellGuild end,
							set = function(key, value) profileDB.tellGuild = value end,
						},
						party = {
							order = 3,
							type = 'toggle',
							name = L["Party"],
							desc = L["Warn party on upcoming battle."],
							get = function() return profileDB.tellParty end,
							set = function(key, value) profileDB.tellParty = value end,
						},
						raid = {
							order = 4,
							type = 'toggle',
							name = L["Raid"],
							desc = L["Warn raid on upcoming battle."],
							get = function() return profileDB.tellRaid end,
							set = function(key, value) profileDB.tellRaid = value end,
						},
					},
				},
				announceTime = {
					order = 3,
					type = 'group',
					name = L["Announce time"],
					desc = L["Choose when to announce the upcoming battle."],
					args = {
						now= {
							order = 1,
							type = 'toggle',
							name = L["Start"],
							desc = L["Warn at battle start."],
							get = function() return profileDB.timeNow end,
							set = function(key, value) profileDB.timeNow = value end,
						},
						five= {
							order = 2,
							type = 'toggle',
							name = "5" .. L[" minutes."],
							desc = L["Warn "] .. "5" .. L[" minutes before battle."],
							get = function() return profileDB.timeFive end,
							set = function(key, value) profileDB.timeFive= value end,
						},
						fifteen= {
							order = 3,
							type = 'toggle',
							name = "15" .. L[" minutes."],
							desc = L["Warn "] .. "15" .. L[" minutes before battle."],
							get = function() return profileDB.timeFifteen end,
							set = function(key, value) profileDB.timeFifteen = value end,
						},
						thirty = {
							order = 4,
							type = 'toggle',
							name = "30" .. L[" minutes."],
							desc = L["Warn "] .. "30" .. L[" minutes before battle."],
							get = function() return profileDB.timeThirty end,
							set = function(key, value) profileDB.timeThirty = value end,
						},
					},
				},
			},
		},
	},
}

function Broker_Wintergrasper:SetupOptions()
	Broker_Wintergrasper.options.args.profile = AceDBOptions:GetOptionsTable(self.db)
	Broker_Wintergrasper.options.args.profile.order = -2

	AceConfig:RegisterOptionsTable("Broker_Wintergrasper", Broker_Wintergrasper.options, {"Broker_Wintergrasper", "BWG"})
	
	self.optionsFrames = {}
	self.optionsFrames.general = AceConfigDialog:AddToBlizOptions("Broker_Wintergrasper", nil, nil, "general")
	self.optionsFrames.profile = AceConfigDialog:AddToBlizOptions("Broker_Wintergrasper", L["Profiles"], "Broker_Wintergrasper", "profile")
end

--------------------------------------------------------------------------------------------------------
--                                       Broker_Wintergrasper core                                    --
--------------------------------------------------------------------------------------------------------

function Broker_Wintergrasper:OnInitialize()
	self.db = AceDB:New("Broker_WintergrasperDB", DATABASE_DEFAULTS, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	
	profileDB = self.db.profile
	
	self:SetupOptions()
end

function Broker_Wintergrasper:OnEnable()
	self:ScheduleRepeatingTimer("MainUpdate", 1)
end

-- LDB object
Broker_Wintergrasper.obj = ldb:NewDataObject("Broker_Wintergrasper", {
	type = "data source",
	label = "Broker_Wintergrasper",
	text = "",
	icon = "Interface\\Icons\\INV_Jewelry_Ring_66",
	OnClick = function(frame, msg)
		if msg == "LeftButton" then
			Broker_Wintergrasper:OnClick()
		elseif msg == "RightButton" then
			Broker_Wintergrasper:ShowConfig()
		end
	end,
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine("Broker_Wintergrasper "..GetAddOnMetadata("Broker_Wintergrasper", "Version"))
		tooltip:AddLine(L["Shift-hint"])
		tooltip:AddLine(L["Right-hint"])
	end,
})

-- Main update function
function Broker_Wintergrasper:MainUpdate()
	if (faction == nil or faction == "unknown" or faction == "battle") then
		self:UpdateIcon()
	end
	if (nextBattle == nil or status == nil) then
		self:UpdateTime()
	end
	local timeText
	if (faction == "battle") then
		timeText = L["Battle"]
	elseif (status == "unknown" and not nextBattle) then
		timeText = L["Unknown"]
	elseif nextBattle then
		local seconds = nextBattle - time()
		if (seconds > 0) then
			timeText = self:GetTimeText(seconds)
			if (seconds > 8990) then
				faction = nil
			end
			if seconds == 1800 and profileDB.timeThirty then
				self:Warn(30)
			elseif seconds == 900 and profileDB.timeFifteen then
				self:Warn(15)
			elseif seconds == 300 and profileDB.timeFive then
				self:Warn(5)
			end
		elseif (seconds == 0 ) then
			if profileDB.timeNow then
				self:Warn(0)
			end
			faction = nil
			nextBattle = nextBattle + 9000
			timeText = self:GetTimeText(seconds)
		else
			faction = nil
			nextBattle = nil
			timeText = L["Unknown"]
		end
	end
	if timeText then
		self.obj.text = "|cffffff00" .. timeText .. "|r"
	end
end

--------------------------------------------------------------------------------------------------------
--                                   Broker_Wintergrasper functions                                   --
--------------------------------------------------------------------------------------------------------

-- Called after profile changed
function Broker_Wintergrasper:OnProfileChanged(event, database, newProfileKey)
	profileDB = database.profile
end

-- Open config window
function Broker_Wintergrasper:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.profile)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.general)
end

-- On click event handler
function Broker_Wintergrasper:OnClick()
	if IsShiftKeyDown() then
		if ChatFrame1EditBox:IsVisible() then
			local message = self:GetAnnounceText(faction, nextBattle)
			ChatFrame1EditBox:Insert(message)
		end
	end
end

-- Update time and status.
function Broker_Wintergrasper:UpdateTime()
	local seconds = GetWintergraspWaitTime()
	if seconds then 
		nextBattle = time() + GetWintergraspWaitTime()
		status = nil
	else
		status = "unknown"
	end
end

-- Search for WG status. Update faction and icon if needed.
function Broker_Wintergrasper:UpdateIcon()
	local aux = 1
	if faction == "battle" then
		aux = GetWintergraspWaitTime()
	end
	if not WorldMapFrame:IsVisible() and aux then
		local continentIndex = GetCurrentMapContinent()
		if not IsInInstance() and continentIndex == 4 then
			local zoneIndex = GetCurrentMapZone()
			SetMapZoom(4)
			local n = GetNumMapLandmarks()
			if n > 0 then
				for i = 1, n do
					local name, _, nr = GetMapLandmarkInfo(i)
					if name == L["Wintergrasp"] then
						if nr == 46 then
							self.obj.icon = "Interface\\Addons\\Broker_Wintergrasper\\alliance"
							faction = "Alliance"
							self:UpdateTime()
							return
						elseif nr == 48 then
							self.obj.icon = "Interface\\Addons\\Broker_Wintergrasper\\horde"
							faction = "Horde"
							self:UpdateTime()
							return
						elseif nr == 101 then
							self.obj.icon = "Interface\\Icons\\Ability_DualWield"
							faction = "battle"
							return
						end
					end
				end			
			end
			SetMapZoom(continentIndex, zoneIndex, 1)
		end
		self.obj.icon = "Interface\\Icons\\INV_Jewelry_Ring_66"
		faction = "unknown"	
	end
end

-- Return Wintergasp wait time in "h:m:s", or "Battle" if match in progress
function Broker_Wintergrasper:GetTimeText(seconds)
	local timeText = ""
	local h = math.floor(seconds / 3600)
	local m = math.floor((seconds % 3600) / 60)
	local s = seconds % 60
	if profileDB.showSeconds then
		if h > 9 then
			timeText = h .. ":"
		elseif h > 0 then
			timeText = "0" .. h .. ":"
		else
			timeText = "00" .. ":"
		end
		if m > 9 then
			timeText = timeText .. m .. ":"
		elseif m > 0 then
			timeText = timeText .. "0" .. m .. ":"
		else
			timeText = timeText .. "00" .. ":"
		end
		if s > 9 then
			timeText = timeText .. s
		elseif s > 0 then
			timeText = timeText .. "0" .. s
		else
			timeText = timeText .. "00"
		end
	elseif seconds > 59 then
		if h > 9 then
			timeText = h .. ":"
		elseif h > 0 then
			timeText = "0" .. h .. ":"
		else
			timeText = "00" .. ":"
		end
		if m > 9 then
			timeText = timeText .. m
		elseif m > 0 then
			timeText = timeText .. "0" .. m
		else
			timeText = timeText .. "00"
		end
	else
		if m > 9 then
			timeText = timeText .. m .. ":"
		elseif m > 0 then
			timeText = timeText .. "0" .. m .. ":"
		else
			timeText = timeText .. "00" .. ":"
		end
		if s > 9 then
			timeText = timeText .. s
		elseif s > 0 then
			timeText = timeText .. "0" .. s
		else
			timeText = timeText .. "00"
		end
	end
	return timeText
end

function Broker_Wintergrasper:GetAnnounceText(faction, nextBattle)
	local message = "[Broker_Wintergrasper] "
	if faction == "Horde" or faction == "Alliance" then
		if faction == UnitFactionGroup("player") then
			message = message .. " " .. L["Defensive battle in"]
		else
			message = message .. " " .. L["Offensive battle in"]
		end
	elseif faction == "battle" then
		message = message .. " " .. L["The battle for Wintergrasp is underway."]
	else
		message = message .. " " .. L["Next battle in"]
	end
	if not (faction == "battle") then
		if nextBattle then
			local seconds = nextBattle - time()
			local timeText = self:GetTimeText(seconds)
			message = message .. " " .. timeText
		else
			message = message .. " " .. L["Unknown"]
		end
	end
	return message
end

function Broker_Wintergrasper:Warn(seconds)
	local numGuild, numParty, numRaid = GetNumGuildMembers(), GetNumPartyMembers(), GetNumRaidMembers()
	local msg = "[Broker_Wintergrasper] "
	
	if seconds == 0 then
		msg = msg .. L["The battle for Wintergrasp has begun."]
	else
		if faction == "Horde" or faction == "Alliance" then
			if faction == UnitFactionGroup("player") then
				msg = msg .. L["Defensive battle in"] .. " "
			else
				msg = msg .. L["Offensive battle in"] .. " "
			end
		else
			msg = msg .. " " .. L["Next battle in"] .. " "
		end
		msg = msg .. seconds .. L[" minutes."]
	end
	
	if profileDB.tellSelf then
		DEFAULT_CHAT_FRAME:AddMessage("|cffff0000" .. msg .. "|r")
	end
	if profileDB.tellGuild and (numGuild > 0) then
		SendChatMessage(msg,"GUILD",nil,nil)
	end
	if profileDB.tellParty and (numParty > 0) then
		SendChatMessage(msg,"PARTY",nil,nil)
	end
	if profileDB.tellRaid and (numRaid > 0) then
		SendChatMessage(msg,"RAID",nil,nil)
	end
end
